local E, L, V, P, G, _ = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB
local CA = E:GetModule('ChannelAlerts')
local CH = E:GetModule('Chat')
local LSM = LibStub('LibSharedMedia-3.0')
local EP = LibStub("LibElvUIPlugin-1.0")
local addon, ns = ...

-- Create incompatibility warning popup
E.PopupDialogs['CA_Incompatible'] = {
	text = 'ElvUI_CustomChannelWarning has been renamed to ElvUI_ChannelAlerts. You still have the old ElvUI_CustomChannelWarning enabled. Click Accept below to disable ElvUI_CustomChannelWarning',
	button1 = ACCEPT,
	OnAccept = function() DisableAddOn('ElvUI_CustomChannelWarning'); ReloadUI(); end,
	timeout = 0,
	whileDead = 1,
	preferredIndex = 3,
}

-- Function which monitors numbered channels 1-10 and plays sound
function CA:MonitorChannels(event, _, author, _, _, _, _, _, channelNumber, ...)
	if author == UnitName('player') then return end -- Don't play sound if the message is from yourself
	if event == 'CHAT_MSG_CHANNEL' and E.db.CA['channel'..channelNumber] ~= 'None' and not CH.SoundPlayed then
		PlaySoundFile(LSM:Fetch('sound', E.db.CA['channel'..channelNumber]), 'Master')
		CH.SoundPlayed = true
		CH.SoundTimer = CH:ScheduleTimer('ThrottleSound', E.db.CA.throttle.channels)
	end
end

-- Function which monitors guild channel and plays sound
function CA:MonitorGuild(event, _, author, ...)
	if author == UnitName('player') then return end -- Don't play sound if the message is from yourself
	if event == 'CHAT_MSG_GUILD' and E.db.CA.guild ~= 'None' and not CH.SoundPlayed then
		PlaySoundFile(LSM:Fetch('sound', E.db.CA.guild), 'Master')
		CH.SoundPlayed = true
		CH.SoundTimer = CH:ScheduleTimer('ThrottleSound', E.db.CA.throttle.guild)
	end
end

-- Function which monitors officer channel and plays sound
function CA:MonitorOfficer(event, _, author, ...)
	if author == UnitName('player') then return end -- Don't play sound if the message is from yourself
	if event == 'CHAT_MSG_OFFICER' and E.db.CA.officer ~= 'None' and not CH.SoundPlayed then
		PlaySoundFile(LSM:Fetch('sound', E.db.CA.officer), 'Master')
		CH.SoundPlayed = true
		CH.SoundTimer = CH:ScheduleTimer('ThrottleSound', E.db.CA.throttle.officer)
	end
end

--Table we store our 1-10 channels in: [1] = ChannelName
CA.Channels = {}

local function buildChannelList(...)
	for i = 1, select("#", ...), 2 do
		local id, name = select(i, ...)
		CA.Channels[id] = name
	end
end

--Function which updates the CA.Channels table
function CA:UpdateChannelTable(event, msg, _, _, _, _, _, _, id, name, ...)
	if msg == "YOU_LEFT" and CA.Channels[id] then
		CA.Channels[id] = nil
	elseif msg == "YOU_JOINED" then
		CA.Channels[id] = name
	end
end

--Do stuff when entering world
function CA:PLAYER_ENTERING_WORLD()
	--Build CA.Channels table
	buildChannelList(GetChannelList())
	
	--Warn about incompatible addon
	if IsAddOnLoaded('ElvUI_CustomChannelWarning') then
		E:StaticPopup_Show('CA_Incompatible')
	end
end

--Function which gets executed when user presses "Update Channels" in config
function CA:UpdateChannelsConfig()
	local group = E.Options.args.blazeplugins.args.CA.args.alerts.args
	for i = 1, 10 do
		local channelName = CA.Channels[i]
		if channelName then
			group['channel'..i].name = channelName..L[' Alert']
			group['channel'..i].hidden = false
		else
			group['channel'..i].hidden = true
		end
	end
end

-- Stuff to do when addon is initialized
function CA:OnInitialize()
	-- Register monitoring functions
	self:RegisterEvent('CHAT_MSG_CHANNEL', 'MonitorChannels')
	self:RegisterEvent('CHAT_MSG_GUILD', 'MonitorGuild')
	self:RegisterEvent('CHAT_MSG_OFFICER', 'MonitorOfficer')
	
	-- Update CA.Channels table on channel joined/left
	self:RegisterEvent('CHAT_MSG_CHANNEL_NOTICE', 'UpdateChannelTable')
	
	self:RegisterEvent('PLAYER_ENTERING_WORLD')
	
	-- Register callback with LibElvUIPlugin
	EP:RegisterPlugin(addon, CA.InsertOptions)
end